defmodule Webmon.Engine do
  require Logger

  use GenServer

  @update_interval 1000

  def start_link(website, on_update: update_callback) do
    GenServer.start_link(__MODULE__, {website, update_callback})
  end

  @impl true
  def init({website, update_callback}) do
    initial_state = %{
      website: website,
      update_callback: update_callback,
      status_code: -1,
      ping: -1
    }
    {:ok, initial_state, {:continue, :finish_init}}
  end

  @impl true
  def handle_continue(:finish_init, state) do
    send(self(), {:ping, state.website})
    {:noreply, state}
  end

  @impl true
  def handle_info({:ping, website}, state) do
    {ms, {:ok, %{status: status}}} = :timer.tc(fn -> ping(website) end)
    state.update_callback.(Map.merge(website, %{status: {:ok, status, ms/1000}}))

    Process.send_after(self(), {:ping, website}, @update_interval)
    {:noreply, state}
  end

  @impl true
  def handle_info(msg, state) do
    Logger.warning("unhandled info: " <> inspect(msg))

    {:noreply, state}
  end

  def ping(website) do
    Req.head(website.url)
  end
end
